<?php
/*
Plugin Name: WooCommerce Bookings Calendar
Plugin URI: https://wordpress.org/plugins/woo-bookings-calendar
Description: Availability Calendar Shortcode for WooCommerce Bookings Extension
Version: 1.0.25
Author: Moises Heberle
Author URI: https://pluggablesoft.com/contact/
Text Domain: woo-bookings-calendar
Domain Path: /i18n/languages/
WC requires at least: 3.2
WC tested up to: 4.5.1
*/

if ( ! defined( 'ABSPATH' ) ) exit;

defined('WBC_BASE_FILE') || define('WBC_BASE_FILE', __FILE__);

add_action('init', 'wbc_init');
add_action('plugins_loaded', 'wbc_plugins_loaded', 10 );
add_filter('mh_wbc_settings', 'wbc_settings');
add_filter('mh_wbc_premium_url', 'wbc_premium_url');
add_action('wp_enqueue_scripts', 'wbc_enqueue_assets');
add_action('woocommerce_after_single_product_summary', 'wbc_after_single_product_summary');

add_shortcode('wbc-calendar', 'wbc_shortcode_calendar');

add_action('wp_ajax_nopriv_wbc_events_load', 'wbc_events_load');
add_action('wp_ajax_wbc_events_load', 'wbc_events_load');

if ( !function_exists('wbc_init') ) {
    function wbc_init() {
        // load Common library
        include_once( 'common/MHCommon.php' );
        MHCommon::initializeV2(
            'woo-bookings-calendar',
            'wbc',
            WBC_BASE_FILE,
            __('WooCommerce Bookings Calendar', 'woo-bookings-calendar')
        );

        if ( !class_exists('WC_Bookings') ) {
            $bookingsUrl = 'https://woocommerce.com/products/woocommerce-bookings/';
            $message = sprintf('Apparently you miss to enable this required extension: <a href="%s" target="_blank">WooCommerce Bookings</a>. Please fix the issue.', $bookingsUrl, $wooUrl);
            $common = MHCommon::getInstance();
            $notice = $common->getNotice();
            $notice->addTempNotice('error', $message);
        }

        if ( !class_exists('WooCommerce') ) {
            $wooUrl = 'https://wordpress.org/plugins/woocommerce/';
            $message = sprintf('Apparently you miss to enable this required plugin: <a href="%s" target="_blank">WooCommerce</a>. Please fix the issue.', $wooUrl, $wooUrl);
            $common = MHCommon::getInstance();
            $notice = $common->getNotice();
            $notice->addTempNotice('error', $message);
        }
        
        // debug for developers only
        if ( !empty($_GET['debug']) ) {
            wbc_debug();
        }

        // allow to use the shortcodes in widgets, in this case [wbc-calendar]
        if ( !has_filter('widget_text', 'do_shortcode') ) {
            add_filter('widget_text', 'do_shortcode');
        }
    }

    function wbc_plugins_loaded() {
        load_plugin_textdomain( 'woo-bookings-calendar', false, plugin_basename( dirname( __FILE__ ) ) . '/i18n/languages' );
    }
    
    function wbc_premium_url() {
        return 'http://gum.co/woocalendar';
    }

    function wbc_settings($arr) {
        $arr['show_events_loading'] = array(
            'label' => __('Display loading while updating calendar events', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'type' => 'checkbox',
            'default' => 'yes',
        );
        $arr['add_event_filter'] = array(
            'label' => __('Enable event filter box in calendar', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'type' => 'checkbox',
            'default' => 'no',
        );
        $arr['customize_colors'] = array(
            'label' => __('Customize bookable product colors', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'type' => 'checkbox',
            'default' => 'no',
        );
        $arr['attach_in_single_page'] = array(
            'label' => __('Attach calendar in single booking page', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'type' => 'checkbox',
            'default' => 'no',
        );
        $arr['disable_back_past'] = array(
            'label' => __('Disable back button to the past time', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'type' => 'checkbox',
            'default' => 'no',
        );
        $arr['calendar_view'] = array(
            'label' => __('Calendar view', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'type' => 'multicheckbox',
            'default' => array('day', 'week', 'month'),
            'options' => array(
                'day' => __('Day', 'woo-bookings-calendar'),
                'week' => __('Week', 'woo-bookings-calendar'),
                'month' => __('Month', 'woo-bookings-calendar'),
                'list_day' => __('Day (list)', 'woo-bookings-calendar'),
                'list_week' => __('Week (list)', 'woo-bookings-calendar'),
                'list_month' => __('Month (list)', 'woo-bookings-calendar'),
            ),
        );            
        $arr['color_available'] = array(
            'label' => __('Bookable color', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'type' => 'color',
            'default' => '#0000CD',
            'depends_on' => 'customize_colors',
        );
        $arr['color_not_bookable'] = array(
            'label' => __('Not bookable color', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'type' => 'color',
            'default' => '#FF0000',
            'depends_on' => 'customize_colors',
        );
        $arr['when_unavailable'] = array(
            'label' => __('When item not bookable', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'type' => 'select',
            'default' => 'show_readonly',
            'options' => array(
                'show_readonly' => __('Display as read-only in calendar', 'woo-bookings-calendar'),
                'hide_display' => __('Hide item from calendar', 'woo-bookings-calendar'),
                'show_normal' => __('Display as bookable item', 'woo-bookings-calendar'),
            ),
        );
        $arr['time_format'] = array(
            'label' => __('Time format', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'type' => 'select',
            'default' => 'HH:mm',
            'options' => array(
                'HH:mm' => sprintf(__('%s - HH:mm'), date('H:m')),
                'h(:mm)a' => sprintf(__('%s - h(:mm)a'), date('ga')),
            ),
        );
        $arr['locale'] = array(
            'label' => __('Calendar language', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'type' => 'select',
            'default' => 'en',
            'options' => array(
                'af'=>'af',
                'ar'=>'ar',
                'ar-dz'=>'ar-dz',
                'ar-kw'=>'ar-kw',
                'ar-ly'=>'ar-ly',
                'ar-ma'=>'ar-ma',
                'ar-sa'=>'ar-sa',
                'ar-tn'=>'ar-tn',
                'bg'=>'bg',
                'bs'=>'bs',
                'ca'=>'ca',
                'cs'=>'cs',
                'da'=>'da',
                'de'=>'de',
                'el'=>'el',
                'en-au'=>'en-au',
                'en'=>'en',
                'en-gb'=>'en-gb',
                'en-nz'=>'en-nz',
                'es'=>'es',
                'et'=>'et',
                'eu'=>'eu',
                'fa'=>'fa',
                'fi'=>'fi',
                'fr-ch'=>'fr-ch',
                'fr'=>'fr',
                'gl'=>'gl',
                'he'=>'he',
                'hi'=>'hi',
                'hr'=>'hr',
                'hu'=>'hu',
                'id'=>'id',
                'is'=>'is',
                'it'=>'it',
                'ja'=>'ja',
                'ka'=>'ka',
                'kk'=>'kk',
                'ko'=>'ko',
                'lb'=>'lb',
                'lt'=>'lt',
                'lv'=>'lv',
                'mk'=>'mk',
                'ms'=>'ms',
                'nb'=>'nb',
                'nl'=>'nl',
                'nn'=>'nn',
                'pl'=>'pl',
                'pt-br'=>'pt-br',
                'pt'=>'pt',
                'ro'=>'ro',
                'ru'=>'ru',
                'sk'=>'sk',
                'sl'=>'sl',
                'sq'=>'sq',
                'sr-cyrl'=>'sr-cyrl',
                'sr'=>'sr',
                'sv'=>'sv',
                'th'=>'th',
                'tr'=>'tr',
                'uk'=>'uk',
                'vi'=>'vi',
                'zh-cn'=>'zh-cn',
                'zh-tw'=>'zh-tw'
            ),
        );
        $arr['calendar_start_date'] = array(
            'label' => __('Fixed calendar start date', 'woo-bookings-calendar'),
            'tab' => __('General', 'woo-bookings-calendar'),
            'default' => '',
            'type' => 'date',
        );

        return $arr;
    }

    function wbc_option($name) {
        return apply_filters('mh_wbc_setting_value', $name);
    }

    function wbc_check_shortcode_inuse() {
        global $post;

        if ( !empty($post->post_content) && has_shortcode($post->post_content, 'wbc-calendar') ) {
            return true;
        }

        return false;
    }

    function wbc_enqueue_assets() {
        global $post;

        // when viewing page with pre selected calendar event
        if ( !empty($_GET['wbc_cal_start']) && !empty($_GET['wbc_cal_end']) ) {
            $start = sanitize_text_field($_GET['wbc_cal_start']);
            $end = sanitize_text_field($_GET['wbc_cal_end']);

            $vars = array(
                'productId' => $post->ID,
                'isSingleProduct' => true,
                'bookingDateTime' => date('H:i', $start),
                'bookingDateDay' => date('d', $start),
                'bookingDateMonth' => date('m', $start),
                'bookingDateYear' => date('Y', $start),
                'bookingEndDay' => date('d', $end),
                'bookingEndMonth' => date('m', $end),
                'bookingEndYear' => date('Y', $end),
            );

            wbc_enqueue_fullcalendar();
            wbc_enqueue_scripts($vars);
        }
    }

    function wbc_enqueue_fullcalendar() {
        wp_enqueue_style('fullcalendar', plugins_url('assets/fullcalendar/fullcalendar.min.css', plugin_basename(WBC_BASE_FILE)));
        wp_enqueue_script('fullcalendar', plugins_url('assets/fullcalendar/fullcalendar.min.js', plugin_basename(WBC_BASE_FILE)), array('moment', 'jquery'));

        if ( wbc_option('locale') != 'en' ) {
            wp_enqueue_script('fc-locales', plugins_url('assets/fullcalendar/locale-all.js', plugin_basename(WBC_BASE_FILE)), array('fullcalendar'));
        }
    }

    function wbc_enqueue_scripts($frontendVars = array()) {
        wp_enqueue_style('woo-bookings-calendar', plugins_url('assets/wbclite.css', plugin_basename(WBC_BASE_FILE)));
        wp_enqueue_script('woo-bookings-calendar', plugins_url('assets/wbclite.js', plugin_basename(WBC_BASE_FILE)), array('jquery'));

        if ( wbc_option('show_events_loading') ) {
            wp_register_script( 'jquery-blockui', WC()->plugin_url() . '/assets/js/jquery-blockui/jquery.blockUI.min.js', array( 'jquery' ));
        }

        wbc_maybe_migrate_settings();

        $displays = wbc_calendar_mode_displays();
        $viewConf = wbc_option('calendar_view');
        $startDate = apply_filters('wbc_calendar_start_date', wbc_option('calendar_start_date'));

        $frontendVars += array(
            'ajaxUrl' => get_admin_url() . 'admin-ajax.php',
            'displayLoading' => wbc_option('show_events_loading'),
            'disableBackPast' => wbc_option('disable_back_past'),
            'fcDefaultView' => current($displays),
            'fcHeaderRight' => implode(',', $displays),
            'dayLabel' => __('day', 'woo-bookings-calendar'),
            'weekLabel' => __('week', 'woo-bookings-calendar'),
            'monthLabel' => __('month', 'woo-bookings-calendar'),
            'listDayLabel' => in_array('day', $viewConf) ? __('list day', 'woo-bookings-calendar') : __('day', 'woo-bookings-calendar'),
            'listWeekLabel' => in_array('week', $viewConf) ? __('list week', 'woo-bookings-calendar') : __('week', 'woo-bookings-calendar'),
            'listMonthLabel' => in_array('month', $viewConf) ? __('list month', 'woo-bookings-calendar') : __('month', 'woo-bookings-calendar'),
            'timeFormat' => wbc_option('time_format'),
            'locale' => wbc_option('locale'),
            'forceStartDate' => $startDate,
            'fcCustomOptions' => apply_filters('wbc_fullcalendar_options', array()),
        );

        $timeZoneString = get_option('timezone_string');
        if ( strlen($timeZoneString) > 0 ) {
            $frontendVars['timezoneStr'] = $timeZoneString;
        }

        $frontendVars = apply_filters('wbc_frontend_vars', $frontendVars);

        wp_localize_script('woo-bookings-calendar', 'WBCVARS', $frontendVars );
    }

    // migrate legacy settings
    function wbc_maybe_migrate_settings() {
        $options = $newOpts = get_option('wbc_settings', array());
        
        if ( array_key_exists('calendar_view_day', $options) ) {
            $newOpts['calendar_view'][] = 'day';
            unset($options['calendar_view_day']);
        }

        if ( array_key_exists('calendar_view_week', $options) ) {
            $newOpts['calendar_view'][] = 'week';
            unset($options['calendar_view_week']);
        }

        if ( array_key_exists('calendar_view_month', $options) ) {
            $newOpts['calendar_view'][] = 'month';
            unset($options['calendar_view_month']);
        }

        if ( array_key_exists('calendar_view_list_day', $options) ) {
            $newOpts['calendar_view'][] = 'list_day';
            unset($options['calendar_view_list_day']);
        }

        if ( array_key_exists('calendar_view_list_week', $options) ) {
            $newOpts['calendar_view'][] = 'list_week';
            unset($options['calendar_view_list_week']);
        }

        if ( array_key_exists('calendar_view_list_month', $options) ) {
            $newOpts['calendar_view'][] = 'list_month';
            unset($options['calendar_view_list_month']);
        }

        if ( $options != $newOpts ) {
            update_option('wbc_settings', $newOpts);
        }
    }

    function wbc_calendar_mode_displays() {
        $display = array();
        $confs = wbc_option('calendar_view');

        foreach ( $confs as $view ) {
            $display[$view] = wbc_translate_view_display($view);
        }

        if ( empty($display) ) {
            $display['day'] = 'agendaDay';
        }

        return apply_filters('wbc_calendar_display', $display);
    }

    function wbc_translate_view_display($view) {
        $view = str_replace('-', '_', $view);

        return str_replace(
            array('list_month', 'list_week', 'list_day', 'week', 'day'),
            array('listMonth', 'listWeek', 'listDay', 'agendaWeek', 'agendaDay'),
            $view
        );
    }

    function wbc_shortcode_calendar( $atts ){
        wbc_enqueue_fullcalendar();
        wbc_enqueue_scripts();

        do_action('wbc_after_shortcode_script_queue');

        $html = '';

        if ( wbc_option('add_event_filter') == 'yes' ) {
            $query = sanitize_text_field(filter_input(INPUT_GET, 'q_event'));

            $html .= '<div class=\'wbc-search\'>';
            $html .= '<form role="search" method="get" class="wbc-search-form">';
            $html .= '<input type="search" class="search-field wbc-search-field" placeholder="' . __('Search events ...') . '" value="'. $query .'" name="q_event">';
            $html .= '</form>';
            $html .= '</div>';
        }

        $extra = ''; // PRO version related variable

        if ( !empty($atts['cat']) ) {
            $extra .= ' data-category="'.$atts['cat'].'"';
        }

        if ( !empty($atts['product']) ) {
            $extra .= ' data-product="'.$atts['product'].'"';
        }

        if ( !empty($atts['default-view']) ) {
            $extra .= ' data-default-view="'.wbc_translate_view_display($atts['default-view']).'"';
        }

        $classes = apply_filters('wbc_calendar_class', array());

        $html .= '<div id="wbc-calendar"'.$extra.' class="'.implode(' ', $classes).'"></div>';

        return $html;
    }

    function wbc_events_load() {
        $start = sanitize_text_field(filter_input(INPUT_POST, 'start'));
        $end = sanitize_text_field(filter_input(INPUT_POST, 'end'));
        $filter = sanitize_text_field(filter_input(INPUT_POST, 'q_event'));

        // Set the client browser timezone when WooCommerce Bookings has this config checked: Display visitor's local time
        // This code needs to be reviewed work with  wbc_build_events_array() function
        // $clientTimeZone = null;
        // if ( wc_should_convert_timezone() ) {
        //     $clientTimeZone = sanitize_text_field(filter_input(INPUT_POST, 'client_timezone'));
        //     if ( !empty($clientTimeZone) ) {
        //         WC_Bookings_Cache::clear_cache();
        //         date_default_timezone_set($clientTimeZone);
        //         wp_cache_set( 'wc_bookings_timezone_string', $clientTimeZone);
        //     }
        // }

        if ( date( 'Ymd', $start ) < date( 'Ymd', current_time( 'timestamp' ) ) ) {
            $start = time();
        }

        $dtStart = date('Ymd', $start);
        $dtEnd = date('Ymd', $end);
        $products = wbc_get_bookable_products($dtStart, $dtEnd, $filter);

        if ( has_filter( 'wbc_build_events_array', 'wbcpro_build_events_array' ) ) {
            $events = apply_filters('wbc_build_events_array', $products, $dtStart, $dtEnd);
        }
        else {
            $events = wbc_build_events_array($products, $dtStart, $dtEnd);
        }

        wp_send_json($events);

        exit;
    }

    function wbc_build_events_array($products, $start, $end, $timeZone = null) {
        $events = array();
        $isPremium = apply_filters('mh_wbc_is_premium', false);

        foreach ( $products as $position => $product ) {
            $bookProd = wc_get_product( $product );

            // debug purpose only
            // if ( $bookProd->get_id() != xxx ) continue; 

            // other advanced types of bookings like Accommodations only supported in premium version
            if ( !$isPremium && !$bookProd->is_type('booking') ) {
                continue;
            }

            $restrictedDays = $bookProd->has_restricted_days() ? $bookProd->get_restricted_days() : array();
            $dateRanges = array();
            $_start = wbc_get_start_range($bookProd, $start);
            $isAccommodation = $bookProd->is_type('accommodation-booking');

            if ( has_filter( 'wbc_get_blocks_in_range', 'wbcpro_get_blocks_in_range' ) ) {
                $blocksInRange = apply_filters('wbc_get_blocks_in_range', $bookProd, strtotime($start), strtotime($end));
            }
            else {
                $blocksInRange = $bookProd->get_blocks_in_range(strtotime($start), strtotime($end));
            }

            if ( !$isAccommodation ) {
                $availableBlocks = wc_bookings_get_time_slots($bookProd, $blocksInRange);
            }

            $availRules = $bookProd->get_availability_rules();
            $excludeRanges = array();

            // When Availability Rules has defined, consider it to create availability blocks
            // TODO This needs to be reviewed. In many cases will not work correct
            if ( !empty($availRules) && !$bookProd->is_type('accommodation-booking') ) {
                foreach ( $availRules as $rule ) {
                    switch ( $rule['type'] ) {
                        case 'custom:daterange':
                            $rule['range'] = wbc_separate_daterange_rule($rule['range']);

                            foreach ( $rule['range'] as $year => $info ) {
                                foreach ( $info as $month => $subInfo ) {
                                    foreach ( $subInfo as $day => $subSubInfo ) {        
                                        $day = str_pad($day, 2, '0', STR_PAD_LEFT);
                                        $month = str_pad($month, 2, '0', STR_PAD_LEFT);
                                        $from = $subSubInfo['from'];
                                        $to = $subSubInfo['to'];

                                        $isBookable = ( $subSubInfo['rule'] === true );
                                        $bStart = strtotime("{$year}-{$month}-{$day} {$from}");
                                        $bEnd = strtotime("{$year}-{$month}-{$day} {$to}");

                                        if ( $isBookable ) {
                                            $dateRanges[] = array($bStart, $bEnd);
                                        }
                                        else {
                                            // when is NOT bookable, the range interval needs to be excluded
                                            $excludeRanges[] = array($bStart, $bEnd);
                                        }
                                    }
                                }
                            }
                            break;
                    }
                }
            }
            else if ( !empty($availableBlocks) ) {
                foreach ( $availableBlocks as $bStart => $info ) {
                    $bStart = wbc_get_start_range($bookProd, $bStart);
                    $bEnd = wbc_get_end_range($bookProd, $bStart);
        
                    $dateRanges[] = array($bStart, $bEnd);
                }
            }
            else if ( !empty($blocksInRange) ) {
                foreach ( $blocksInRange as $bStart ) {
                    $bStart = wbc_get_start_range($bookProd, $bStart);
                    $bEnd = wbc_get_end_range($bookProd, $bStart);
                    $dateRanges[] = array($bStart, $bEnd);
                }
            }

            // make works with time/hour blocks
            if ( empty($dateRanges) ) {
                foreach ( $blocksInRange as $time ) {
                    $bStart = wbc_get_start_range($bookProd, $time);
                    $bEnd = wbc_get_end_range($bookProd, $time);

                    // ensure the block is available
                    if ( !count($bookProd->get_blocks_in_range($bStart, $bEnd)) ) {
                        continue;
                    }

                    $dateRanges[] = array($bStart, $bEnd);
                }
            }
            
            if ( !empty($excludeRanges) ) {
                $dateRanges = wbc_exclude_ranges($dateRanges, $excludeRanges);
            }

            $whenUnavailable = wbc_option('when_unavailable');
            $availables = null;
            $alwaysFetchAvailables = apply_filters('wbc_always_get_availables', false);
            $dateRanges = apply_filters('wbc_filter_date_ranges', $dateRanges, $bookProd);

            foreach ( $dateRanges as $ranges ) {
                list($_start, $_end) = $ranges;

                if ( !empty($restrictedDays) && !in_array(date('w', $_start), $restrictedDays) ) {
                    continue;
                }

                // check if the item has at least 1 quantity available or any other error that not allow to book in the date
                $availError = null;
                if ( $whenUnavailable != 'show_normal' || $alwaysFetchAvailables ) {
                    $resource = null;

                    if ( $bookProd->has_resources() ) {
                        $resources = $bookProd->get_resources($product->ID);
                        if (!empty($resources[0])) {
                            $resourceId = $resources[0]->ID;
                            $resource = new WC_Product_Booking_Resource( $resourceId );
                        }
                    }

                    if ( has_filter( 'wbc_loop_blocks_availability', 'wbcpro_loop_blocks_availability' ) ) {
                        $availables = apply_filters('wbc_loop_blocks_availability', $bookProd, $_start, $_end, $resource);
                    }
                    else {
                        $availables = $bookProd->get_blocks_availability($_start, $_end, 1, $resource);
                    }
        
                    if ( is_wp_error( $availables ) || is_bool($availables) && $availables === false ) {
                        // not include this range when the configuration tell to hide item when unavailable (hide_display)
                        if ( $whenUnavailable == 'hide_display' ) {
                            continue;
                        }

                        // sets the error message when configuration tell to show item as readonly when unavailable (show_readonly)
                        // $availError = $availables->get_error_message();
                        $availError = __('BOOKED', 'woo-bookings-calendar');
                        $availables = 0;
                    }
                }

                $args = array('wbc_cal_start' => $_start, 'wbc_cal_end' => $_end);
                $url = add_query_arg($args, get_permalink($product));
                $color = empty($availError) ? wbc_color_available($bookProd, $position) : wbc_color_unavailable($bookProd);

                $eventTitle = $product->post_title;
                if ( !empty($availError) ) {
                    $eventTitle .= ' (' . $availError . ')';
                }

                $event = array();
                $event['title'] = $eventTitle;
                $event['start'] = date('Y-m-d H:i', $_start);
                $event['end'] = date('Y-m-d H:i', $_end);
                $event['post_id'] = $product->ID;
                $event['color'] = $color;
                $event['availables'] = $availables;
                $event['type'] = $bookProd->get_type();

                if ( empty($availError) ) {
                    $event['url'] = $url;
                }

                $events[] = apply_filters('wbc_event_row', $event, $product, $bookProd, $availables);
            }
        }

        return apply_filters('wbc_filter_events', array_filter($events));
    }

    function wbc_get_start_range($bookProd, $time) {
        $unit = $bookProd->get_duration_unit();
        $timeFiltered = apply_filters('wbc_filter_start_range', $time, $unit, $bookProd);
        
        if ( $timeFiltered != $time ) {
            $time = $timeFiltered;
        }
        else {
            switch($unit) {
                case 'hour':
                    $time = strtotime(date('Y-m-d H:i', $time));
                    break;
    
                case 'day':
                    $time = strtotime(date('Y-m-d H:i', $time));
                    break;
    
                case 'month':
                    $time = strtotime(date('Y-m-d 00:00', $time));
                    break;
            }
        }

        return $time;
    }

    function wbc_get_end_range($bookProd, $start) {
        $endFiltered = apply_filters('wbc_filter_end_range', $start, $bookProd);

        if ( $endFiltered != $start ) {
            $end = $endFiltered;
        }
        else {
            $duration = $bookProd->get_duration();
            $unit = $bookProd->get_duration_unit();
            $end = strtotime("+{$duration} {$unit}", $start);
        }

        return $end;
    }

    // this function check all date ranges to find the intersections that need to be excluded accordingly from Availability
    // configuration defined as Bookable = No
    function wbc_exclude_ranges($dateRanges, $excludeRanges) {
        $newRanges = array();
        
        foreach ($dateRanges as $dRange) {
            list($rStart, $rEnd) = $dRange;
            $foundIntersection = false;

            foreach ( $excludeRanges as $eRange ) {
                list($eStart, $eEnd) = $eRange;

                if ( $eStart > $rStart && $eStart < $rEnd ) {
                    $newRanges[] = array($rStart, $eStart-60);
                    $newRanges[] = array($eEnd+60, $rEnd-60);
                    $foundIntersection = true;
                }
            }

            if ( !$foundIntersection ) {
                $newRanges[] = $dRange;
            }
        }

        return $newRanges;
    }

    function wbc_color_available($bookProd, $position) {
        $color = apply_filters('wbc_color_available', $bookProd);

        if ( is_string($color) ) {
            return $color;
        }

        if ( wbc_option('customize_colors') == 'yes' ) {
            return wbc_option('color_available');
        }

        // randomize colors
        $colors = array(
            'royalblue',
            'olive',
            'royalblue',
            'darkcyan',
            'green',
            'orange',
        );

        if ( isset($colors[$position]) ) {
            return $colors[$position];
        }

        return 'blue';
    }

    function wbc_color_unavailable($bookProd) {
        $color = apply_filters('wbc_color_unavailable', $bookProd);

        if ( is_string($color) ) {
            return $color;
        }

        if ( wbc_option('customize_colors') == 'yes' ) {
            return wbc_option('color_not_bookable');
        }

        return 'red';
    }

    function wbc_get_bookable_products($start = null, $end = null, $filter = null){
        $args = apply_filters( 'get_booking_products_args', array(
            'post_status'    => 'publish',
            'post_type'      => 'product',
            's'              => $filter,
            'posts_per_page' => -1,
            'tax_query'      => array(
                array(
                    'taxonomy' => 'product_type',
                    'field'    => 'slug',
                    'terms'    => array('booking'),
                ),
            ),
            'suppress_filters' => true,
        ) );

        $productIds = !empty($_POST['product_ids']) ? explode(',', $_POST['product_ids']) : null;

        if ( !empty($productIds) ) {
            $args['post__in'] = $productIds;
        }

        $args = apply_filters( 'wbc_booking_products_args', $args);
        $results = get_posts($args);

        return $results;
    }

    // in many use cases it desired to show the start and to time separately in calendar
    // when configured "Range type = Date range with time" rule
    // so this function define all start time and end time as same
    function wbc_separate_daterange_rule($rule) {
        $firstFrom = current(current(current($rule)));
        $firstFrom = $firstFrom['from'];

        $lastTo = end(end(end($rule)));
        $lastTo = $lastTo['to'];

        foreach ( $rule as $year => $info ) {
            foreach ( $info as $month => $subInfo ) {
                foreach ( $subInfo as $day => $subSubInfo ) {
                    if ( !$subSubInfo['rule'] ) {
                        continue;
                    }

                    $rule[$year][$month][$day]['from'] = $firstFrom;
                    $rule[$year][$month][$day]['to'] = $lastTo;
                }
            }
        }
        return $rule;
    }

    function wbc_debug() {
        $start = '20190401';
        $end = '20190420';
        $products = wbc_get_bookable_products($start, $end);
        $events = wbc_build_events_array($products, $start, $end);

        // var_dump($events); exit;

        var_dump(date('H:i'));
        var_dump( wc_booking_get_timezone_string() );
        var_dump( wc_should_convert_timezone() );
    }

    function wbc_debug_dateranges($dateRanges) {
        foreach ( $dateRanges as $dates ) {
            var_dump(date('d/m/Y H:i', $dates[0]).' to '.date('d/m/Y H:i', $dates[1]));
        }

        exit;
    }

    function wbc_after_single_product_summary() {
        global $product;

        if ( !$product->is_type(array('booking', 'accommodation-booking')) || ( wbc_option('attach_in_single_page') != 'yes' ) ) {
            return;
        }

        $atts = array('product' => $product->get_id());
        echo wbc_shortcode_calendar($atts);
    }
}

